#!/usr/local/bin/python2.4

import queue
import dpkt
import getopt
import sys

def getkey(ip):
	tcp = ip.data
	return (ip.src, tcp.sport, ip.dst, tcp.dport)

def calcackbytes(acka, ackb):
	r = ackb - acka
	if r < -2**32 + 2**31:	# 1 gig window size for SACK
		r = ackb - acka + 2 ** 32
	return r

class AckCalc:
	def __init__(self):
		self.cons = {}

	def dotcpack(self, ip):
		assert isinstance(ip.data, dpkt.tcp.TCP) and ip.data.flags & dpkt.tcp.TH_ACK

		ret = 0
		tcp = ip.data
		key = getkey(ip)

		try:
			lastack = self.cons[key]
			ackbytes = calcackbytes(lastack, tcp.ack)
			if ackbytes > 0:
				ret = ackbytes
			self.cons[key] = tcp.ack
		except KeyError:
			# no connection, create one
			self.cons[key] = tcp.ack

		return ret

	def __call__(self, datum):
		ret = 0
		buf, addr = datum
		ip = dpkt.ip.IP(buf)
		if isinstance(ip.data, dpkt.tcp.TCP):
			if ip.data.flags & dpkt.tcp.TH_FIN:
				try:
					key = getkey(ip)
					del self.cons[key]
				except KeyError:
					pass
			elif ip.data.flags & dpkt.tcp.TH_ACK:
				ret = self.dotcpack(ip)

		return ret

def genfun(datum):
	ret = 0
	buf, addr = datum
	ip = dpkt.ip.IP(buf)
	if isinstance(ip.data, dpkt.tcp.TCP):
		ret = len(ip.data.data)

	return ret

def usage():
	sys.stderr.write('''Usage: ratelimit.py -p <divertport> -m <maxbytesperqueue> -a <bytespersec>  | -u <bytespersec> ...\n''')

def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "a:hm:p:u:", ["help", ])
	except getopt.GetoptError:
		# print help information and exit:
		usage()
		sys.exit(2)

	dvrtprt = None
	qs = []
	maxbytes = None
	for o, a in opts:
		if o in ("-h", "--help"):
			usage()
			sys.exit()
		elif o in ("-p", ):
			dvrtprt = int(a)
		elif o in ("-m", ):
			maxbytes = int(a)
		elif o in ("-a", "-u", ):
			if o in ("-a", ):
				fun = AckCalc()
			elif o in ("-u", ):
				fun = genfun
			else:
				raise NotImplementedError, "no class for: %s" % o
			qs.append((maxbytes, fun, int(a)))

	if not dvrtprt or not qs:
		usage()
		sys.exit(2)

	d = queue.DivertQueue(dvrtprt, block = False)
	for maxbytes, fun, a in qs:
		d.append(queue.UnitQueue(fun, a, maxbytes))

	while True:
		d.checkandhandle()

if __name__ == '__main__':
	main()
